@@ -215,7 +215,7 @@ class PlaybackProvider:
 
     def is_live(self, uri):
         """
-        Decide if the URI should be threated as a live stream or not.
+        Decide if the URI should be treated as a live stream or not.
 
         *MAY be reimplemented by subclass.*
 
@@ -228,9 +228,24 @@ class PlaybackProvider:
         """
         return False
 
+    def should_download(self, uri):
+        """
+        Attempt progressive download buffering for the URI or not.
+
+        *MAY be reimplemented by subclass.*
+
+        When streaming a fixed length file, the entire file can be buffered
+        to improve playback performance.
+
+        :param uri: the URI
+        :type uri: string
+        :rtype: bool
+        """
+        return False
+
     def change_track(self, track):
         """
-        Swith to provided track.
+        Switch to provided track.
 
         *MAY be reimplemented by subclass.*
 
@@ -250,7 +265,11 @@ class PlaybackProvider:
             logger.debug("Backend translated URI from %s to %s", track.uri, uri)
         if not uri:
             return False
-        self.audio.set_uri(uri, live_stream=self.is_live(uri)).get()
+        self.audio.set_uri(
+            uri,
+            live_stream=self.is_live(uri),
+            download=self.should_download(uri),
+        ).get()
         return True
 
     def resume(self):
