@@ -1,5 +1,3 @@
-from __future__ import absolute_import, unicode_literals
-
 import copy
 import logging
 import time
@@ -10,9 +8,7 @@ from mopidy.internal.models import HistoryState, HistoryTrack
 logger = logging.getLogger(__name__)
 
 
-class HistoryController(object):
-    pykka_traversable = True
-
+class HistoryController:
     def __init__(self):
         self._history = []
 
@@ -25,17 +21,18 @@ class HistoryController(object):
         :type track: :class:`mopidy.models.Track`
         """
         if not isinstance(track, models.Track):
-            raise TypeError('Only Track objects can be added to the history')
+            raise TypeError("Only Track objects can be added to the history")
 
         timestamp = int(time.time() * 1000)
 
         name_parts = []
         if track.artists:
             name_parts.append(
-                ', '.join([artist.name for artist in track.artists]))
+                ", ".join(artist.name for artist in track.artists)
+            )
         if track.name is not None:
             name_parts.append(track.name)
-        name = ' - '.join(name_parts)
+        name = " - ".join(name_parts)
         ref = models.Ref.track(uri=track.uri, name=name)
 
         self._history.insert(0, (timestamp, ref))
@@ -64,14 +61,13 @@ class HistoryController(object):
         count = 1
         history_list = []
         for timestamp, track in self._history:
-            history_list.append(
-                HistoryTrack(timestamp=timestamp, track=track))
+            history_list.append(HistoryTrack(timestamp=timestamp, track=track))
             count += 1
             if count_max < count:
-                logger.info('Limiting history to %s tracks', count_max)
+                logger.info("Limiting history to %s tracks", count_max)
                 break
         return HistoryState(history=history_list)
 
     def _load_state(self, state, coverage):
-        if state and 'history' in coverage:
+        if state and "history" in coverage:
             self._history = [(h.timestamp, h.track) for h in state.history]
