@@ -1,25 +1,28 @@
-from __future__ import absolute_import, unicode_literals
-
 import collections
 import logging
-import os
+from collections.abc import Mapping
 
 import pkg_resources
 
-from mopidy import config as config_lib, exceptions
+from mopidy import config as config_lib
+from mopidy import exceptions
 from mopidy.internal import path
 
-
 logger = logging.getLogger(__name__)
 
 
-_extension_data_fields = ['extension', 'entry_point', 'config_schema',
-                          'config_defaults', 'command']
+_extension_data_fields = [
+    "extension",
+    "entry_point",
+    "config_schema",
+    "config_defaults",
+    "command",
+]
 
-ExtensionData = collections.namedtuple('ExtensionData', _extension_data_fields)
+ExtensionData = collections.namedtuple("ExtensionData", _extension_data_fields)
 
 
-class Extension(object):
+class Extension:
 
     """Base class for Mopidy extensions"""
 
@@ -49,7 +52,8 @@ class Extension(object):
         :returns: bytes or unicode
         """
         raise NotImplementedError(
-            'Add at least a config section with "enabled = true"')
+            'Add at least a config section with "enabled = true"'
+        )
 
     def get_config_schema(self):
         """The extension's config validation schema
@@ -57,7 +61,7 @@ class Extension(object):
         :returns: :class:`~mopidy.config.schemas.ConfigSchema`
         """
         schema = config_lib.ConfigSchema(self.ext_name)
-        schema['enabled'] = config_lib.Boolean()
+        schema["enabled"] = config_lib.Boolean()
         return schema
 
     @classmethod
@@ -67,11 +71,13 @@ class Extension(object):
         Use this directory to cache data that can safely be thrown away.
 
         :param config: the Mopidy config object
-        :return: string
+        :return: pathlib.Path
         """
-        assert cls.ext_name is not None
-        cache_dir_path = bytes(os.path.join(config['core']['cache_dir'],
-                                            cls.ext_name))
+        if cls.ext_name is None:
+            raise AssertionError
+        cache_dir_path = (
+            path.expand_path(config["core"]["cache_dir"]) / cls.ext_name
+        )
         path.get_or_create_dir(cache_dir_path)
         return cache_dir_path
 
@@ -80,11 +86,13 @@ class Extension(object):
         """Get or create configuration directory for the extension.
 
         :param config: the Mopidy config object
-        :return: string
+        :return: pathlib.Path
         """
-        assert cls.ext_name is not None
-        config_dir_path = bytes(os.path.join(config['core']['config_dir'],
-                                             cls.ext_name))
+        if cls.ext_name is None:
+            raise AssertionError
+        config_dir_path = (
+            path.expand_path(config["core"]["config_dir"]) / cls.ext_name
+        )
         path.get_or_create_dir(config_dir_path)
         return config_dir_path
 
@@ -95,11 +103,13 @@ class Extension(object):
         Use this directory to store data that should be persistent.
 
         :param config: the Mopidy config object
-        :returns: string
+        :returns: pathlib.Path
         """
-        assert cls.ext_name is not None
-        data_dir_path = bytes(os.path.join(config['core']['data_dir'],
-                                           cls.ext_name))
+        if cls.ext_name is None:
+            raise AssertionError
+        data_dir_path = (
+            path.expand_path(config["core"]["data_dir"]) / cls.ext_name
+        )
         path.get_or_create_dir(data_dir_path)
         return data_dir_path
 
@@ -148,7 +158,7 @@ class Extension(object):
         raise NotImplementedError
 
 
-class Registry(collections.Mapping):
+class Registry(Mapping):
 
     """Registry of components provided by Mopidy extensions.
 
@@ -159,13 +169,13 @@ class Registry(collections.Mapping):
 
     - ``backend`` is used for Mopidy backend classes.
     - ``frontend`` is used for Mopidy frontend classes.
-    - ``local:library`` is used for Mopidy-Local libraries.
 
     Extensions can use the registry for allow other to extend the extension
-    itself. For example the ``Mopidy-Local`` use the ``local:library`` key to
-    allow other extensions to register library providers for ``Mopidy-Local``
-    to use. Extensions should namespace custom keys with the extension's
-    :attr:`~Extension.ext_name`, e.g. ``local:foo`` or ``http:bar``.
+    itself. For example the ``Mopidy-Local`` historically used the
+    ``local:library`` key to allow other extensions to register library
+    providers for ``Mopidy-Local`` to use. Extensions should namespace
+    custom keys with the extension's :attr:`~Extension.ext_name`,
+    e.g. ``local:foo`` or ``http:bar``.
     """
 
     def __init__(self):
@@ -196,21 +206,25 @@ def load_extensions():
 
     installed_extensions = []
 
-    for entry_point in pkg_resources.iter_entry_points('mopidy.ext'):
-        logger.debug('Loading entry point: %s', entry_point)
+    for entry_point in pkg_resources.iter_entry_points("mopidy.ext"):
+        logger.debug("Loading entry point: %s", entry_point)
         try:
             extension_class = entry_point.resolve()
         except Exception as e:
-            logger.exception("Failed to load extension %s: %s" % (
-                entry_point.name, e))
+            logger.exception(
+                f"Failed to load extension {entry_point.name}: {e}"
+            )
             continue
 
         try:
             if not issubclass(extension_class, Extension):
                 raise TypeError  # issubclass raises TypeError on non-class
         except TypeError:
-            logger.error('Entry point %s did not contain a valid extension'
-                         'class: %r', entry_point.name, extension_class)
+            logger.error(
+                "Entry point %s did not contain a valid extension" "class: %r",
+                entry_point.name,
+                extension_class,
+            )
             continue
 
         try:
@@ -219,18 +233,25 @@ def load_extensions():
             default_config = extension.get_default_config()
             command = extension.get_command()
         except Exception:
-            logger.exception('Setup of extension from entry point %s failed, '
-                             'ignoring extension.', entry_point.name)
+            logger.exception(
+                "Setup of extension from entry point %s failed, "
+                "ignoring extension.",
+                entry_point.name,
+            )
             continue
 
-        installed_extensions.append(ExtensionData(
-            extension, entry_point, config_schema, default_config, command))
+        installed_extensions.append(
+            ExtensionData(
+                extension, entry_point, config_schema, default_config, command
+            )
+        )
 
         logger.debug(
-            'Loaded extension: %s %s', extension.dist_name, extension.version)
+            "Loaded extension: %s %s", extension.dist_name, extension.version
+        )
 
     names = (ed.extension.ext_name for ed in installed_extensions)
-    logger.debug('Discovered extensions: %s', ', '.join(names))
+    logger.debug("Discovered extensions: %s", ", ".join(names))
     return installed_extensions
 
 
@@ -241,63 +262,82 @@ def validate_extension_data(data):
     :returns: if extension should be run
     """
 
-    logger.debug('Validating extension: %s', data.extension.ext_name)
+    logger.debug("Validating extension: %s", data.extension.ext_name)
 
     if data.extension.ext_name != data.entry_point.name:
         logger.warning(
-            'Disabled extension %(ep)s: entry point name (%(ep)s) '
-            'does not match extension name (%(ext)s)',
-            {'ep': data.entry_point.name, 'ext': data.extension.ext_name})
+            "Disabled extension %(ep)s: entry point name (%(ep)s) "
+            "does not match extension name (%(ext)s)",
+            {"ep": data.entry_point.name, "ext": data.extension.ext_name},
+        )
         return False
 
     try:
         data.entry_point.require()
-    except pkg_resources.DistributionNotFound as ex:
+    except pkg_resources.DistributionNotFound as exc:
         logger.info(
-            'Disabled extension %s: Dependency %s not found',
-            data.extension.ext_name, ex)
+            "Disabled extension %s: Dependency %s not found",
+            data.extension.ext_name,
+            exc,
+        )
         return False
-    except pkg_resources.VersionConflict as ex:
-        if len(ex.args) == 2:
-            found, required = ex.args
+    except pkg_resources.VersionConflict as exc:
+        if len(exc.args) == 2:
+            found, required = exc.args
             logger.info(
-                'Disabled extension %s: %s required, but found %s at %s',
-                data.extension.ext_name, required, found, found.location)
+                "Disabled extension %s: %s required, but found %s at %s",
+                data.extension.ext_name,
+                required,
+                found,
+                found.location,
+            )
         else:
             logger.info(
-                'Disabled extension %s: %s', data.extension.ext_name, ex)
+                "Disabled extension %s: %s", data.extension.ext_name, exc
+            )
         return False
 
     try:
         data.extension.validate_environment()
-    except exceptions.ExtensionError as ex:
-        logger.info(
-            'Disabled extension %s: %s', data.extension.ext_name, ex.message)
+    except exceptions.ExtensionError as exc:
+        logger.info("Disabled extension %s: %s", data.extension.ext_name, exc)
         return False
     except Exception:
-        logger.exception('Validating extension %s failed with an exception.',
-                         data.extension.ext_name)
+        logger.exception(
+            "Validating extension %s failed with an exception.",
+            data.extension.ext_name,
+        )
         return False
 
     if not data.config_schema:
-        logger.error('Extension %s does not have a config schema, disabling.',
-                     data.extension.ext_name)
+        logger.error(
+            "Extension %s does not have a config schema, disabling.",
+            data.extension.ext_name,
+        )
         return False
-    elif not isinstance(data.config_schema.get('enabled'), config_lib.Boolean):
-        logger.error('Extension %s does not have the required "enabled" config'
-                     ' option, disabling.', data.extension.ext_name)
+    elif not isinstance(data.config_schema.get("enabled"), config_lib.Boolean):
+        logger.error(
+            'Extension %s does not have the required "enabled" config'
+            " option, disabling.",
+            data.extension.ext_name,
+        )
         return False
 
     for key, value in data.config_schema.items():
         if not isinstance(value, config_lib.ConfigValue):
-            logger.error('Extension %s config schema contains an invalid value'
-                         ' for the option "%s", disabling.',
-                         data.extension.ext_name, key)
+            logger.error(
+                "Extension %s config schema contains an invalid value"
+                ' for the option "%s", disabling.',
+                data.extension.ext_name,
+                key,
+            )
             return False
 
     if not data.config_defaults:
-        logger.error('Extension %s does not have a default config, disabling.',
-                     data.extension.ext_name)
+        logger.error(
+            "Extension %s does not have a default config, disabling.",
+            data.extension.ext_name,
+        )
         return False
 
     return True
